use std::{fs::File, io::BufReader};

use log::warn;
use serde::Deserialize;

use super::{deserialize, table_directory::{FontTable, TableDirectoryRecord}};

#[derive(Debug, Deserialize, Default)]
pub struct Version {
	major: u16,
	minor: u16
}

#[derive(Debug, Deserialize)]
pub struct FontHeader {
	#[serde(skip)]
	version: Version,
	
	#[serde(skip)]
	font_revision: Version,

	checksum_adjust: u32,
	magic_number: u32,
	flags: u16,
	units_per_em: u16,

	created: i64,
	modified: i64,

	xmin: i16,
	ymin: i16,
	xmax: i16,
	ymax: i16,

	mac_style: u16,
	lowest_rec_ppem: u16,

	font_direction_hint: i16,
	index_to_loc_hint: i16,
	glyph_data_format: i16
}

impl FontTable for FontHeader {
	const TAG: &'static str = "head";

	fn decode(reader: BufReader<File>, _: &TableDirectoryRecord) -> bincode::Result<Self>
		where Self: Sized
	{
		let version = deserialize::<_, Version>(reader.get_ref())?;
		if version.major != 1 && version.minor != 0 {
			warn!("Font version is not 1.0");
		}

		let font_revision = deserialize::<_, Version>(reader.get_ref())?;

		let mut font_header: FontHeader = deserialize(reader.get_ref())?;
		font_header.version = version;
		font_header.font_revision = font_revision;

		Ok(font_header)
	}
}
